#! /usr/bin/perl -w


#-------------------------------------------------------------
#PredSL - PREDiction of Subcellular Location from the N-terminal Sequence
#Copyright (C) 2005 , Evangelia I. Petsalaki
#Dept. of Cell Biology and Biophysics
#Faculty of Biology
#University of Athens
#-------------------------------------------------------------

#    This file is part of PredSL.
#
#    PredSL is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    PredSL is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with PredSL; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA




use strict;
#this program needs a file with the sequences in fasta format and 1 if they are plant sequences or 0 otherwise
my $bindir="./";#Here we determine where the bin directiry is
#First we determine whether the sequences are plant (1) or nonplant (0) and if we want the 
#graphical representation of the cleavage site prediction
 my $pid=$$; 
my $plant;	#1 an plant 0 an nonplant
my $graph;	#flag to determine if you want the graphic output (1) or not (0)

if (defined $ARGV[1])
{
	$plant=$ARGV[1];
}

else
{
	print "Give 1 if plant proteins and 0 if nonplant ";
	$plant=<STDIN>;
	chomp $plant;
}
if (defined $ARGV[2])
{
	$graph=$ARGV[2];
}
else
{
	$graph=0;
}

###############################################################################################################

#We read the sequences in fasta format and we run predsl for each one

my $entrfile=$ARGV[0]; # We give the input file

open (ENTRY,"$entrfile")||die "ERROR:Cannot open file with entries!!";

my $sequence; # This variable will contain the protein's sequence
my $door=0; # flag to run predsl after it has gathered the first sequence
my $id_a="";

while (<ENTRY>)
{
	if ($_=~/^>(.*?) / || $_=~/^>(.*)/ )
	{
		#$sq="";
		if ($door==1)	#turning flag on
		{&predsl;  		#we run predsl foreach sequence
			#&fasta;
			$sequence="";

		}
		my $id=$1;
		$id=~s/\s//;
		$id_a=$id;
		$door=1;
	}
	
	else
	{
		chomp;
		my $seq=$_;
		$sequence=$sequence.$seq;
		$sequence=~s/\s//g;
		$sequence=~s/\n//g;
	}
	
}
&predsl;
#&fasta;
close ENTRY;
sub fasta 
{
	print $id_a,"\n",$sequence,"\n";  # subroutine to check that it gathers all sequences
}

#This is subroutine predsl
sub predsl
{

##################################Initialization of all global variables, arrays etc ##########################################################################################

my $entrfile;	#sequences you want to test in fasta format. #>ID\s{2}location\s{2}cleavage\nsequence//.
my $count;	#to discriminate if the program will perform a self check or not
my $pad_sc;	#################
my @npin;	#contains variables for $short
my $short;	#chlor or mitoch
my $f;		#position - $f, pou arxizei to parathiro
my $t;		#position + $t, pou teleionei to parathiro
my $range;	#size of window for the first neural network
my $inp;	#########gets corresponding vars for each neural network
my $vars;	##########how many inputs in the first neural network +1
my $in;		#how many inputs in the neural network
my @ID;		#saves all IDs
my @subloc;	#saves all subcellular locations (1,2,3,0)
my @cleavage;	#saves all cleavage sites
my @sequence;	#saves all sequences
my $cutoffnp;	#cutoff to find approximate cleavage site
my $varscls;	##########how many inputs in the cleavage site neural network +1	
my $chlres;	#hash containing ID and score from prediction for the chloroplast procedure
my $mitres;	#hash containing ID and score from prediction for the mitochondrion procedure
my @AA; 	#Amino-acids
my @ind;	#index, codes for each amino acid
my $null;	#variable containing 20 zeros (0) to use if the window is outside thw sequence
my $nulla;
my $hiddennp;	#number of hidden units of the first neural network
my %hydro;      #hash of hydrophobicity index (Kyte-Doolittle)
my @tst_a;	#the input variables for the first neural network (first 100 residues)
my @pts_a;	#saves the scores from the prediction of the first neural network
my @pts_chlor;	#saves the scores from the chloroplast prediction of the first neural network
my @pts_mitoch;	#saves the scores from the mitochondrial prediction of the first neural network
my $chlor_prednp; #saves cleavage site of cTP
my $mitoch_prednp; #saves cleavage site of mTP
my @sp_prednp;	   #saves all cleavage sites of SP
my $sp_prednp;	   #saves cleavage site of SP for each sequence
my $pts_chlor;	  #saves output from the third neural network that predicts the existance of a cTP
my $pts_mitoch;	  #saves output from the third neural network that predicts the existance of an mTP
my @predisi;	#saves all predisi scores
my $predisi;	#saves predisi scores for each sequence
my @signal;	#Y if predisi considers it has an SP N if not
my $signal;	#Y if predisi considers it has an SP N if not
my @IDpredisi;	#gathers ids from predisi results
my $tr_ccy;	#gets scores from Markov chain program chlor-cyt
my $tr_mcy;	#gets scores from Markov chain program mit-cyt
my $tr_mc;	#gets scores from Markov chain program chlor-mit
my $tr_mccy;	#gets scores from Markov chain program chlor&mit-cyt
my $tr_m_cysp;	#gets scores from Markov chain program mit-sp&cyt
my $tr_msp;	#gets scores from Markov chain program mit-sp
my $tr_mcsp;	#gets scores from Markov chain program chlor&mit-sp
my $tr_ch_mcy;	#gets scores from Markov chain program chlor-cyt&mit
my $hmm_chlor;	#gets the score from the hmmres_chlor
my $hmm_mitoch;	#gets the score from the hmmres_mitoch
my $fin_pred; #results from the final network
my $thyl;	#1 if there id an lTP 0 if not
my $ltp;	#cleavage site of lTP
my $pts_tst_chlor; #gets the input for the neural network that takes as input the 100 scores from the first one(chloroplast)
my $pts_tst_mitoch;#gets the input for the neural network that takes as input the 100 scores from the first one(mitochondrion)


#####################################################################################################################################################


####################################### Arxikopoiisi metavliton #####################################################################################

$entrfile=$ARGV[0];
@AA=qw(A C D E F G H I K L M N P Q R S T V W Y);	#Amino-acids			
@ind=("0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1","0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0","0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0","0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0",
	"0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0",
	"0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0",
	"0\t0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0","0\t1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0", 
	"1\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0"); #index, codes for each amino acid
$null="0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t";
$nulla="0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0\t0";
%hydro=('A'=> '1.8', 'C' => '2.5', 'D' => '-3.5', 'E' => '-3.5', 'F' => '2.8', 'G' => '-0.4', 
'H'=> '-3.2', 'I' => '4.5', 'K' => '-3.9', 'L' => '3.8', 'M' => '1.9', 'N'=> '-3.5', 'P' => '-1.6',
 'Q' => '-3.5','R'=> '-4.5', 'S' => '-0.8', 'T' => '-0.7', 'V' => '4.2', 'W' => '-0.9', 'Y' => '-1.3');  #hash of hydrophobicity index (Kyte-Doolittle)

$hiddennp=4; #how many hidden units in the first network
$count=0;

if ($plant==1)
{
	@npin=("chlor","mitoch");
}

if ($plant==0)
{
	@npin=("mitoch");
}

#####################################################################################################################################################

foreach my $lo (0..$#npin)
{
	$short=$npin[$lo];
	if ($short eq "chlor")				#Define the variables for the chloroplast procedure
	{										
		$f=20;					#position -$f ..position+$t gives the size of the window 
		$t=6;					#for the neural network of the cleavage site prediction
		$range=27;
		$vars = 1101;				#Number of variables for the first neural network
		$in = 1100;				#Number of inputs for the first neural network	
		$cutoffnp=0.24;				#cutoff for calculation of approximate cleavage site
		$varscls=541;				#Number of variables for the cleavage site neural network			
	}		  
	
	
	if ($short eq "mitoch")				#Define the variables for the mitochondrion procedure
	{										
		$f=12;					#position -$f ..position+$t gives the size of the window 				
		$t=8;					#for the neural network of the cleavage site prediction
		$range=17;
		$vars = 701;				#Number of variables for the first neural network				
		$in = 700;				#Number of inputs for the first neural network					
		$cutoffnp=0.08;				#cutoff for calculation of approximate cleavage site
		$varscls=421;				#Number of variables for the cleavage site neural network				
	}
	
################################################ Creating the inputs of the first neural network #######################################################

		@tst_a=();#Initializing the array with the inputs for the first neural network

		my @temp=split(//,$sequence);
		my $sqlen=$#temp;
		my @code=();
		
		
		for(my $k=0; $k <= $#temp; $k++)				
		{	
			my $q=0;							
			for(my $j=0; $j <= 19 ; $j++)				 	
			{							
				if ($temp[$k] eq $AA[$j])			
				{						
					$code[$k]=$ind[$j];
					$q=1;			#We correspond each amino-acid residue of the sequence to its index
					last;
				}
				elsif ($j==19 && $q==0)
				{
					$code[$k]=$nulla;
				}						
			}							
		}
		for (my $position=0 ; $position <= 99 ; $position++)		#The following procedure is done for the first 100 residues
		{	
			my $tst="";						#initializing a temporary variable to gather inputs							
			my $beg=$position - $range;				#begining of window in each position
			my $fin=$position + $range;
			$tst=$id_a."_".$position."\t";	
						
			if ($beg < 0 && $fin <= $sqlen	)			#If the position is <$range
			{													
				my $a=-$beg;					
				my $n=$null x $a;				#We print null until there is a residue in the position of the window
				$tst.=$n;
				foreach my $j (0..$fin)
				{							
					$tst.=$code[$j]."\t";			#We print the amino acids' code for the particular window
				}						
			}							
			elsif ($fin > $sqlen && $beg >= 0 && $beg <= $sqlen)			#If position >sqlen
			{	
				foreach my $j ($beg..$sqlen)
				{				
					$tst.=$code[$j]."\t";
				}
				my $p=$fin-$sqlen;
				my $n=$null x $p;
				$tst.=$n;
			}
			elsif ($beg > $sqlen)
			{
				foreach my $k ($beg..$fin)
				{
					$tst.=$null;
				}
			}
			elsif ($beg >= 0 && $fin <= $sqlen)			#If position >sqlen and > $range
			{ 
				foreach my $j ($beg..$fin)
				{				
					$tst.=$code[$j]."\t";
				}
			}			
				
			elsif ($beg < 0 && $fin > $sqlen)			#for small sequences that do not fall in the above categories
			{
				my $a=-$beg;
				my $n=$null x $a;
				$tst.=$n;				
				foreach my $j (0..$sqlen)
				{						
					$tst.=$code[$j]."\t";		
				}			
				my $p=$fin-$sqlen;
				my $na=$null x $p;
				$tst.=$na;
			}
			$tst.="0";
			push @tst_a,$tst;
	
		}					
	
##################################################### Get scores-predictions from the first neural network ##########################################

	my @posit;	#position in neural networks
	my @wts;	#weights for each neural network
	
	if ($short eq "chlor")
	{
		open (FILE,"$bindir/tables/chlor.wts")||die "File chlor.wts does not exist";
		while (<FILE>)
		{
			if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
			{
				push @posit, $1;
				push @wts, $2;
			
			}
		}
		close FILE;
	}

	if ($short eq "mitoch")
	{
		if ($plant==1)
		{
			open (FILE,"$bindir/tables/mitoch.wts")||die "File mitoch.wts does not exist";
			while (<FILE>)
			{
				if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
				{
					push @posit, $1;
					push @wts, $2;
			
				}
			}
			close FILE;
		}
		if ($plant==0)
		{
			open (FILE,"$bindir/tables/mit_np.wts")||die "File chlor.wts does not exist";
			while (<FILE>)
			{
				if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
				{
					push @posit, $1;
					push @wts, $2;
			
				}
			}
			close FILE;
			
		}
	}
	@pts_a=(); #initialization of the array that will gather the scores of the prediction from the first neural network

	foreach my $tst (0..$#tst_a)		#we calculate the oputput scores from the neural network
	{
		my @score=();
		$tst_a[$tst]=~s/\t\n//;
		my @temp=split (/\t/,$tst_a[$tst]);

		for (my $j=0 ; $j<4 ; $j++)	#for each of the 4 hidden units
		{
			foreach my $k(0..$#temp-1)
			{
				if ($posit[$k+$j*$vars]==0)
				{
					$score[$j]=$wts[$k+$j*$vars];
				}
				else 
				{
					$score[$j]=$score[$j]+($temp[$k]*$wts[$k+$j*$vars]);
				}
			}

		}

		foreach my $k(0..$#score)
		{
			$score[$k]=((1/(1+exp(-$score[$k])))-(1/2));
		}

		my $output=$wts[$#wts-4];
		foreach my $k(0..$#score)		#for the output unit
		{
			$output=$output+$score[$k]*$wts[$#wts-4+$k+1];

		}
		$output=(1/(1+exp(-$output)));
		$output=(int($output*1000000))/1000000;
		my $pts_f= $temp[0]."\t".$output."\n";
		push @pts_a, $pts_f;
		if ($short eq "chlor")
		{
			push @pts_chlor, $pts_f;
		}
		if ($short eq "mitoch")
		{
			push @pts_mitoch, $pts_f;
		}
		$pts_f="";
		$output=0;
	}

#####################################################################################################################################################	


##############################################Calculation of approximate cleavage site #####################################################

	my $aa=99; 									
	my $cls;
					
	my $IDA;
	my @position;
	my @score;
	
	#$entry="";
	foreach my $pts (0..$#pts_a)								#We read the results from the neural network and estimate the approximate cleavage site							
	{
		if ($pts_a[$pts]=~m/(.*)_([0-9]{1,2})\t([0,1]\.\d*)/)
		{
		my $id=$1;			
		push @position,$2;
		push @score,$3;
		
		my $e=$2;
		my $te=0;
			if ($e == $aa)
			{
				foreach my $k(0..$aa)
				{
					if ($score[$k] < $cutoffnp)
					{
						$IDA=$id;
						$cls=$position[$k];
						$te=1;
						last;
					}
					elsif ($k==$aa && $te==0)
					{
						$IDA=$id;
						$cls=$position[$aa];
					}
				}
				$id="";
				@position=();
				@score=();
				$te=0;
				
			}
		}
	}
	
	close FILE;									

############################################################################################################################################

#####################################################Make test file for cleavage site prediction ####################################

	my @tst_b=(); #Initialization of array with input nodes for cleavage site prediction network
	#open (OUTB,">$bindir/$short.cls.tst")||die "Cannot write";
									
			if ($id_a eq $IDA)
			{

				my $startpos = $cls-20;	#We take a window of 41 residues around the previously estimated cleavage site
				my $finpos = $cls+20;

				my @code=();

				my @temp=split(//,$sequence);
				my $sqlen=$#temp;

				for(my $z=0; $z <= $#temp; $z++)
				{
					for(my $j=0; $j <= 19 ; $j++)
					{
						my $q=0;
						if ($temp[$z] eq $AA[$j])
						{
							$code[$z]=$ind[$j];		#correspond the residues to their codes	
							$q=1;
							last;
						}
						elsif ($j==19 && $q==0)
						{
							$code[$z]=$nulla;
						}			
					}	
				}
				for (my $position=$startpos ; $position <= $finpos ; $position++)	#foreach position of the window get the inputs
				{
					my $tst=""; #initialization of temporary variable to gather inputs for each position
					my $beg=$position-($f);		#begining of window
					my $fin=$position+($t);		#end of window
					#print OUTB $id_a,"_",$position,"\t";
					$tst.=$id_a."_".$position."\t";
					if ($beg < 0 && $fin <= $#temp && $fin >= 0)
					{

						my $a=-$beg;
						#print  OUTB $null x $a;
						my $n=$null x $a;
						$tst.=$n;
						foreach my $j (0..$fin)
						{
							#print OUTB $code[$j],"\t";
							$tst.=$code[$j]."\t";
						}
					}

					if ($fin > $#temp && $beg >= 0 && $beg <= $#temp)
					{

						foreach my $j ($beg..$#temp)
						{
						#print OUTB $code[$j],"\t";
						$tst.=$code[$j]."\t";
						}
						my $p=$fin-$#temp;
						#print OUTB $null x $p;
						my $n=$null x $p;
						$tst.=$n;
					}
					elsif ($beg > $#temp)
					{
						foreach my $k ($beg..$fin)
						{
							#print OUTB $null;
							$tst.=$null;
						}
					}

					if ($fin < 0)
					{
						foreach my $k ($beg..$fin)
						{
							#print OUTB $null;
							$tst.=$null;
						}
					}
					if ($fin > $#temp && $beg < 0)
					{

						my $a=-$beg;
						#print  OUTB $null x $a;
						my $n=$null x $a;
						$tst.=$n;
						foreach my $j (0..$#temp)
						{
							#print OUTB $code[$j],"\t";
							$tst.=$code[$j]."\t";
						}
						my $p=$fin-$#temp;
						#print OUTB $null x $p;
						my $na=$null x $p;
						$tst.=$na;
					}

					if ($beg >= 0 && $fin <= $#temp)
					{ 

						foreach my $j ($beg..$fin)
						{
							#print OUTB $code[$j],"\t";
							$tst.=$code[$j]."\t";
						}
					}

					#print OUTB "0\n";
					$tst.="0";
					push @tst_b,$tst;
					$tst="";

				}
				@temp=();
				@code=();			
				#$sq="";
				$startpos=0;
				$finpos=0;
			}	

		


	#close OUTB;


############################################################################################################################################

#########################################################Calculating scores for the cleavage site prediction################################

	my $hidun=9; #hidden units of the neural network for the cleavage site prediction
	my $inp=$varscls; #number of variables given to network
	@score=();
	@posit=();
	my @pts_b=(); #Initialization of variable that gathers the output scores from the neural network for the cleavage site prediction

	foreach my $a(1..5)
	{
		#open (OUTA,">$bindir.$short.$a.pts")||die "Cannot open $short.$a.pts file" ;

		if ($plant==1)
		{
			if ($short eq "chlor")
			{
				
					open (FILE,"$bindir/tables/chlor.$a.wts")||die "Cannot open $short.$a.wts file" ;
					while (<FILE>)
					{
						if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
						{
							push @posit, $1;
							push @wts, $2;
			
						}
					}
					close FILE;
				
						
			}#end of if chlor block

			if ($short eq "mitoch")
			{
				
					open (FILE,"$bindir/tables/mitoch.$a.wts")||die "Cannot open $short.$a.wts file" ;
					while (<FILE>)
					{
						if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
						{
							push @posit, $1;
							push @wts, $2;
			
						}
					}
					close FILE;					
			
				
			}#end of if mitoch	
		}#end of if plant=1
		if ($plant==0)
		{
				
					open (FILE,"$bindir/tables/mit_np.$a.wts")||die "Cannot open mit_np.$a.wts file" ;
					while (<FILE>)
					{
						if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
						{
							push @posit, $1;
							push @wts, $2;
			
						}
					}
					close FILE;
					
				
		}#end of if plant=0
		
		foreach my $tst (0..$#tst_b)
		{
			my @temp=split(/\t/,$tst_b[$tst]);

			for (my $j=0 ; $j<$hidun ; $j++)
			{
				foreach my $k(0..$#temp-1)
				{
					if ($posit[$k+$j*$inp]==0)
					{
						$score[$j]=$wts[$k+$j*$inp];
					}
					else 
					{
						$score[$j]=$score[$j]+($temp[$k]*$wts[$k+$j*$inp]);
					}
				}

			}

			foreach my $k(0..$#score)
			{
				$score[$k]=((1/(1+exp(-$score[$k])))-(1/2));
				#print $temp[0]=$score[$i],"\n";
			}

			my $output=$wts[$#wts-$hidun];
			foreach my $k(0..$#score)
			{
				$output=$output+$score[$k]*$wts[$#wts-$hidun+$k+1];

			}
			$output=(1/(1+exp(-$output)));
			$output=(int($output*1000000))/1000000;
			#print OUTA $temp[0],"\t",$output,"\n";
			my $pts=$temp[0]."\t".$output."\n";
			push @pts_b,$pts;
			#print $pts;
			@score=();
			$output=0;
			@temp=();

		}
		#close OUTA;
		@wts=();
	}#END OF A=1-5
	
############################################################################################################################################

#####################################################get average of cleavage site scores ###################################################

	my $cutoff=0.0;
	my @IDr=();
	my @positionr=();
	my @scorer=();
	my $a=0;
	my @medpredr=();
	my @predictionr=();
	my @ptsresults=(); #Initialization of array that gathers average scores from the 5 clsite prediction-neural networks

		foreach my $pts_c (0..$#pts_b)
		{
				if ($pts_b[$pts_c]=~m/(.*)_(-?[0-9]{1,3})\t([0,1]\.\d*)/)
				{	
					push @IDr,$1;					#For each file of the cleavage site prediction we
					push @positionr,$2;				#gather the id and the prediction in order to
					push @predictionr,$3;				#get the avegage

				}
		}

	
	my $w= @IDr;
	my $qu= $w/5;
	

	foreach my $k(0..$qu-1)
	{	
		foreach my $j(0..4)
		{
			
			$a=$a+$predictionr[$k+$j*$qu];			
		}

	$medpredr[$k]=$a/5;								#Calculates average of scores
	$a=0;
	
	}

	
	#open (PTSPROCESS,">$bindir/$short.ptsresults")||die "Can't open file1 to write the results of the processing of scores!";

	
	foreach my $k(0..$qu-1)
	{	
		if ($medpredr[$k] >= $cutoff )
		{	
			my $ptsr=$IDr[$k]."\t".$positionr[$k]."\t".$medpredr[$k]."\n";
			push @ptsresults, $ptsr;
		}
	}

	#close PTSPROCESS;
	

	@IDr=();
	@positionr=();
	@scorer=();
	

############################################################################################################################################

########################################### Prediction of cleavage site ####################################################################

	my $IDw;
	@position=();						
	@score=();
	my $true;
	my $hs;
	my $stdev=20; #APOKLISI APO PRAGMATIKO CLS 

	#open (NPPRED,">$bindir/$short.prednp")||die "Cannot write predictions!!";


	foreach my $ptr (0..$#ptsresults)
	{
		if ($ptsresults[$ptr]=~/(.*)\t(-?\d*)\t(0\.\d*)\n/)
		{	
			$IDw=$1;
			push @position,$2;						
			push @score,$3;
		}
	}

	my $negative=0;
	my $g=0;
	my $maxscore=$score[0];
 	my $bpos=0;
	my $z;
	my $trpos=0;
	


	foreach my $k(1 .. $#score)
	{
		if ($score[$k] > $maxscore)
		{
			$maxscore=$score[$k];
			$bpos=$position[$k];
									
		}
		
		if ($k==$#score)
		{
			
			$hs=$bpos;
			#print NPPRED $IDw,"\t", $bpos,"\t",$maxscore,"\t",$trpos,"\n";
			$maxscore=$score[$k];
			$bpos=$position[$k];
		}
	}
	#close NPPRED;
	if ($short eq "chlor")
	{
		$chlor_prednp=$hs;
	}
	if ($short eq "mitoch")
	{
		$mitoch_prednp=$hs;
	}
	
	
#######################################################Gets the average scores from the signals predicted################################

		my @positionq=();
		my @scoreq=();
		$aa=99;
		my $sc=0;
		my $finsc=0;

			foreach my $pts (0..$#pts_a)#Calculates the average score based on the first neural network from position 1 to the predicted cleavage site 
			{
				if ($pts_a[$pts]=~m/(.*)_([0-9]{1,2})\t([0,1]\.\d*)/) 
				{
					#my $idq=$1;			
					push @positionq,$2;
					push @scoreq,$3;
				}
				
			}#end of foreach loop			
				if ($hs<=99)
				{
					foreach my $x(0..$hs)
					{
						$sc=$sc+$scoreq[$x];
						#print "$y\t$hs{$y}\t$scoreq[$x]\n";
					}
				}
				else
				{
					foreach my $x(0..99)
					{
						$sc=$sc+$scoreq[$x];
						#print "$y\t$hs{$y}\t$scoreq[$x]\n";
					}
				}
				if ($hs)
				{
					$finsc = $sc/$hs;
				}
				else
				{
					#print "Problem\n";
					$finsc = 0;
				}
				#print $short,"\t",$idq,"\t",$hs{$y},"\t",$finsc,"\n";
				if ($short eq "chlor")
				{
					$chlres=$finsc;
				}
				if ($short eq "mitoch")
				{
					$mitres=$finsc;
				}
				$sc=0;
				$finsc=0;

##############################################Making input file with the 100 initial scores #########################################################


#my $flag=0;


my $pts_tst;	
	#open (PTSOUT,">$bindir/$short.pts.tst")||die "Cannot write in $short.pts.tst";
	foreach my $pts (0..$#pts_a)
	{
		if (($pts_a[$pts]=~/^(.*)_(\d{2})\t(\d\.\d*)\n/)||($pts_a[$pts]=~/^(.*)_(\d)\t(\d\.\d*)\n/))
		{#print $1;
		
			if ($2==0)
			{
				#print PTSOUT "$1\t";
				$pts_tst.="$1\t";
			}
		
			#print PTSOUT "$3\t";
			$pts_tst.="$3\t";
			if ($2==99)
			{
				#print PTSOUT "0\n";
				$pts_tst.="0\n";	
			}
		}
	}	
if ($short eq "chlor")
{
	$pts_tst_chlor=$pts_tst;
}
if ($short eq "mitoch")
{
	$pts_tst_mitoch=$pts_tst;
}
	#close PTSOUT;

#print $pts_tst,"\n";

#########################################################################################################################################################

#############################################Getting scores chlor_pts and mitoch_pts####################################################################
my $pts_res;
@posit=(0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , 11 , 12 , 13 , 14 , 15 , 16 , 17 , 18 , 19 , 20 , 
21 , 22 , 23 , 24 , 25 , 26 , 27 , 28 , 29 , 30 , 31 , 32 , 33 , 34 , 35 , 36 , 37 , 38 , 39 , 40 , 
41 , 42 , 43 , 44 , 45 , 46 , 47 , 48 , 49 , 50 , 51 , 52 , 53 , 54 , 55 , 56 , 57 , 58 , 59 , 60 , 
61 , 62 , 63 , 64 , 65 , 66 , 67 , 68 , 69 , 70 , 71 , 72 , 73 , 74 , 75 , 76 , 77 , 78 , 79 , 80 , 
81 , 82 , 83 , 84 , 85 , 86 , 87 , 88 , 89 , 90 , 91 , 92 , 93 , 94 , 95 , 96 , 97 , 98 , 99 , 100);

##ptschl
if ($plant==1)
{
	my @pts=();
	if ($short eq "chlor")
	{
		open (FILE,"$bindir/tables/pts_chl.wts")||die "File chlor.wts does not exist";
		while (<FILE>)
		{
			if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
			{
				
				push @pts, $2;
			
			}
		}
		close FILE;
	}
	if ($short eq "mitoch")
	{
		open (FILE,"$bindir/tables/pts_mit.wts")||die "File chlor.wts does not exist";
		while (<FILE>)
		{
			if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
			{
				
				push @pts, $2;
			
			}
		}
		close FILE;
	}
my $score=0;
my $output=0;
my $hnp=0;
$inp=101;

#open (FILEV,">$bindir/pts_chlor.pts")||die "Cannot write in pts_chlor";
#foreach my $p (0..$#pts_a)
#{
	chomp $pts_tst;
	my @temp=split (/\t/,$pts_tst);

	
		foreach my $k(0..$#temp-1)
		{
			if ($posit[$k]==0)
			{
				$score=$pts[$k];
			}
			else 
			{
				$score=$score+($temp[$k]*$pts[$k]);
			}
		}
		
	
	$output=(1/(1+exp(-$score)));
	$output=(int($output*1000000))/1000000;#print $temp[$#temp],"\n";
	$pts_res=$temp[0]."\t".$output."\t".$temp[$#temp]."\n";
	if ($short eq "chlor")
	{
		$pts_chlor=$output;
	}
	if ($short eq "mitoch")
	{
		$pts_mitoch=$output;
	}
	$output=0;
	@temp=();
	$score=0;
#}



}#end of if plant

if ($plant==0)
{
	my @pts_np=();
	open (FILE,"$bindir/tables/pts_mit_np.wts")||die "File chlor.wts does not exist";
		while (<FILE>)
		{
			if ($_=~/^\t\d*\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
			{
				
				push @pts_np, $2;
			
			}
		}
		close FILE;

		my $score=0;
		my $output=0;
		my $hnp=0;
		my $inp=101;
		
		#foreach my $p (0..$#pts_a)
		#{
			chomp $pts_tst;
			my @temp=split (/\t/,$pts_tst);
		
				foreach my $k(0..$#temp-1)
				{
					if ($posit[$k]==0)
					{
						$score=$pts_np[$k];
					}
					else 
					{
						$score=$score+($temp[$k]*$pts_np[$k]);
					}
				}
				
			
			
			$output=(1/(1+exp(-$score)));
			$output=(int($output*1000000))/1000000;
			$pts_res= $temp[0]."\t".$output."\t".$temp[$#temp]."\n";
			$pts_mitoch=$output;
			$output=0;
		#}


}#end of if plant==0 loop

########################################################################################

	
}#end of chloroplast and mitochondrion procedure

#####################################################################################################################################################################

open (OUT,">$bindir/jspp_$pid")||die;

	print OUT ">$id_a\n";
	my @temp=();
	@temp= split (//,$sequence);
	foreach my $i (0..$#temp)
	{
		print OUT $temp[$i];
		if ($i%60 ==0 && $i>=60)
		{
			print OUT "\n";
			
		}
	}
	print OUT "\n";

close OUT;

`java JSPP $bindir/eukarya.smx $bindir/jspp_$pid $bindir/predisi_$pid`;
#requires folder jspp and JSPP.class

open (FILEB,"$bindir/predisi_$pid")||die "predisi_res";

while (<FILEB>)
{
	#$_=~s/ //g;
	if ($_=~/^(.*)\t(\d*)\t([Y,N])\t(\d\.\d*E?)\n/)
	{
		#push @IDpredisi,$1;
		#push @sp_prednp,$2;
		$sp_prednp=$2;
		$signal=$3;
		#push @signal,$3;
		if ($4=~/E/)
		{
			#push @predisi,0.0;
			$predisi=0.0;
		}
		else
		{
			#push @predisi,$4;
			$predisi=$4;
		}
	}
}
close FILEB;
#foreach my $i (0..$#predisi)
#{
#	if ($IDpredisi[$i]=~/$id_a/)
#	{
#		$sp_prednp=$sp_prednp[$i];
#		$signal=$signal[$i];
#		$predisi=$predisi[$i];
#	}
#}
#####################################################################################################################################################################

#################################################Procedure of getting scores from the Markov chains program ##################################################################

if ($plant==1)
{
	open (FILE, "$bindir/tables/TRAINED_ccy")||die;
my %TRAINED_ccy = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_ccy{$1}=$2;
	}
}
close FILE;
$pad_sc=0;
my $res=130;
		my @temp=split(//,$sequence);
		#print $#temp,"\n";
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<129 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;
				last;
					
			}
			my $a=$temp[$j].$temp[$j+1];#print STDERR $id_a,"\t",$a,"\n";
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_ccy{$a};
			}
			$a="";
		}
		$pad_sc=$pad_sc/$res;
		#print PADELISA $id_a,"\t",$pad_sc,"\n";		
		$tr_ccy=$pad_sc;
		$pad_sc=0;
#print $tr_ccy,"\n";

open (FILE, "$bindir/tables/TRAINED_mcy")||die;
my %TRAINED_mcy = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_mcy{$1}=$2;
	}
}
close FILE;

$pad_sc=0;
$res=80;
		
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<79 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;
				last;	
				
			}
			my $a=$temp[$j].$temp[$j+1];
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_mcy{$a};
			}
			$a="";
		}
		$pad_sc=$pad_sc/$res;
		$tr_mcy=$pad_sc;
		$pad_sc=0;
#print $tr_mcy,"\n";
open (FILE, "$bindir/tables/TRAINED_mc")||die;
my %TRAINED_mc = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_mc{$1}=$2;
	}
}
close FILE;


$pad_sc=0;
$res=130;
		
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<129 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j	;
				last;
				
			}
			my $a=$temp[$j].$temp[$j+1];
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_mc{$a};
			}
			$a="";
		}
		$pad_sc/=$res;
		$tr_mc=$pad_sc;
		$pad_sc=0;

open (FILE, "$bindir/tables/TRAINED_ch_mcy")||die;
my %TRAINED_ch_mcy = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_ch_mcy{$1}=$2;
	}
}
close FILE;

$pad_sc=0;
$res=130;
		
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<129 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;
				last;	
				
			}
			my $a=$temp[$j].$temp[$j+1];
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_ch_mcy{$a};
			}
			$a="";
		}
		$pad_sc/=$res;
		$tr_ch_mcy=$pad_sc;
		$pad_sc=0;

open (FILE, "$bindir/tables/TRAINED_mccy")||die;
my %TRAINED_mccy = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_mccy{$1}=$2;
	}
}
close FILE;

$pad_sc=0;
$res=80;		
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<79 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;
				last;	
				
			}
			my $a=$temp[$j].$temp[$j+1];
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_mccy{$a};
			}
			$a="";
		}
		$pad_sc/=$res;
		$tr_mccy=$pad_sc;
		$pad_sc=0;

open (FILE, "$bindir/tables/TRAINED_mcsp")||die;
my %TRAINED_mcsp = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_mcsp{$1}=$2;
	}
}
close FILE;
$pad_sc=0;
$res=130;
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<129 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;	
				last;
				
			}
			my $a=$temp[$j].$temp[$j+1];
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_mcsp{$a};
			}
			$a="";
		}
		$pad_sc/=$res;
		$tr_mcsp=$pad_sc;
		$pad_sc=0;

}#end of if plant=1 

if ($plant==0)
{
	open (FILE, "$bindir/tables/TRAINED_mcy_np")||die;
my %TRAINED_mcy = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_mcy{$1}=$2;
	}
}
close FILE;
	$pad_sc=0;
	my $res=80;
		my @temp=split(//,$sequence);
		
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<79 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;
				last;	
			}
			my $a=$temp[$j].$temp[$j+1];
			
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_mcy{$a};
			}
			#print STDERR $a,"\t",$pad_sc,"\n";
			$a="";
		}
		$pad_sc/=$res;
		$tr_mcy=$pad_sc;
		$pad_sc=0;
	
open (FILE, "$bindir/tables/TRAINED_msp_np")||die;
my %TRAINED_msp = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_msp{$1}=$2;
	}
}
close FILE;

	$pad_sc=0;
	$res=130;	
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<129 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;
				last;	
			}
			my $a=$temp[$j].$temp[$j+1];
			
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_msp{$a};
			}
			#print STDERR $a,"\t",$pad_sc,"\n";
			$a="";
		}
		$pad_sc/=$res;
		$tr_msp=$pad_sc;
		$pad_sc=0;
	
	open (FILE, "$bindir/tables/TRAINED_m_cysp_np")||die;
my %TRAINED_m_cysp = ();
while (<FILE>)
{
	if ($_=~/\'(\w{2})\'.*\=\>.*\'(.*)\'/)
	{
		$TRAINED_m_cysp{$1}=$2;
	}
}
close FILE;

	$pad_sc=0;
	$res=130;	
		
		#print $id,"\t",$sequence,"\n";
		for (my $j=0 ; $j<129 ; $j++)
		{
			if (!$temp[$j+1])
			{$res=$j;
				last;	
			}
			my $a=$temp[$j].$temp[$j+1];			
			if ($a!~/X/)
			{
				$pad_sc=$pad_sc+$TRAINED_m_cysp{$a};
			}
			#print STDERR $a,"\t",$pad_sc,"\n";
			$a="";
		}
		$pad_sc/=$res;
		$tr_m_cysp=$pad_sc;
		$pad_sc=0;

}#end of if plant==0

################################################################################################################################################

################################################# HMMER - searching for the profiles ###############################################################################################
open (OUT,">$bindir/temp$pid.0-120")||die;

	print OUT ">$id_a\n";
	@temp=();
	@temp= split (//,$sequence);
	foreach my $i (0..119)
	{
		
		if ($i == 59 )
		{
			print OUT "\n";
		}
		if ( $i ==118 && $#temp >118)
		{
			print OUT "\n";
		}
		if ($temp[$i])
		{
			print OUT $temp[$i];
		}
	}
	print OUT "\n";

close OUT;

open (OUT,">$bindir/temp$pid.30-120")||die;
	print OUT ">$id_a\n";
	
	@temp=();
	@temp= split (//,$sequence);
	foreach my $i (30..119)
	{
		
		if ($i == 89 )
		{
			print OUT "\n";
		}
		if ($temp[$i])
		{
			print OUT $temp[$i];
		}
	}
	print OUT "\n";

close OUT;

if ($plant==1)
{
	`$bindir/hmmsearch.exe -E 10000 -T -1000 $bindir/chlor.hmm $bindir/temp$pid.0-120 > $bindir/hmmres_chlor$pid`;
	`$bindir/hmmsearch.exe -E 10000 -T -1000 $bindir/lumen.hmm $bindir/temp$pid.30-120 > $bindir/hmmres_lumen$pid`;
	`$bindir/hmmsearch.exe -E 10000 -T -1000 $bindir/lumen_c.hmm $bindir/temp$pid.30-120 > $bindir/hmmres_lumen_c$pid`;
	`$bindir/hmmsearch.exe -E 10000 -T -1000 $bindir/lumen_d.hmm $bindir/temp$pid.30-120 > $bindir/hmmres_lumen_d$pid`;
}
`$bindir/hmmsearch.exe -E 10000 -T -1000 $bindir/mitoch.hmm $bindir/temp$pid.0-120 > $bindir/hmmres_mitoch$pid`;

if ($plant)
{
open (IN,"$bindir/hmmres_chlor$pid")||die "hmmchlor";
{
	while(<IN>)
	{
		if ($_=~/(.*?) \s*\d\/\d*\s*\d*\s*\d*\s[\[,\.][\.\]]\s*\d*\s*\d*\s[\[,\.][\.\]]\s*(-?\d*\.?\d?)\s.*/)
		{
			$hmm_chlor=$2;
			
		}
	}
}
close IN;
}
open (IN,"$bindir/hmmres_mitoch$pid")||die "hmmmitoch";
{
	while(<IN>)
	{
		if ($_=~/(.*?) \s*\d\/\d*\s*\d*\s*\d*\s[\[,\.][\.\]]\s*\d*\s*\d*\s[\[,\.][\.\]]\s*(-?\d*\.?\d?)\s.*/)
		{
			$hmm_mitoch=$2;
		}
	}
}
close IN;
 
 
##############################################################################################################################

my $final;
if ($plant==1)
{
	$final="$id_a\t$chlres\t$mitres\t$predisi\t$pts_chlor\t$pts_mitoch\t$tr_ccy\t$tr_mcy\t$tr_ch_mcy\t$tr_mc\t$tr_mccy\t$tr_mcsp\t$hmm_chlor\t$hmm_mitoch\t0\t0\t0";
}
if ($plant==0)
{
	$final="$id_a\t$mitres\t$predisi\t$pts_mitoch\t$tr_mcy\t$tr_msp\t$tr_m_cysp\t$hmm_mitoch\t0\t0";
}
#print $final,"\n";
#print $tr_ccy,"\n";


#####################################################make prediction  ###################################################################

if ($plant==1)
{
my @hu=();
my @posit=();
my @wts=();

	open (FILE,"$bindir/tables/plant_train.wts")||die "File chlor.wts does not exist";
	while (<FILE>)
	{
		if ($_=~/^\t(\d*)\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
		{
			push @hu,$1;
			push @posit, $2;
			push @wts, $3;
			
		}
	}
		close FILE;
	


my @score=();
my $output1=0;
my $output2=0;
my $output3=0;
my $hnp=6; 
my $inp=14;

#open (FINOUT,">$bindir/final_prediction")||die "Cannot open test file";
	my @temp =split (/\t/,$final);

	for (my $j=0 ; $j<$hnp ; $j++)
	{
		foreach my $k(0..$#temp-3)
		{
			if ($posit[$k+$j*$inp]==0)
			{
				$score[$j]=$wts[$k+$j*$inp];
			}
			else 
			{
				$score[$j]=$score[$j]+($temp[$k]*$wts[$k+$j*$inp]);
			}
		}
		
	}
	
	foreach my $k(0..$#score)
	{
		$score[$k]=((1/(1+exp(-$score[$k])))-(1/2));
		#print $_[0]=$score[$k],"\n";
	}
	
	
	
	$output1=$wts[$#wts+1-($hnp+1)*3];
	$output2=$wts[$#wts+1-($hnp+1)*2];
	$output3=$wts[$#wts+1-($hnp+1)];
	foreach my $k(0..$#score)
	{
		$output1=$output1+$score[$k]*$wts[$#wts+2-($hnp+1)*3+$k];
		
	}
	foreach my $k(0..$#score)
	{
		$output2=$output2+$score[$k]*$wts[$#wts+2-($hnp+1)*2+$k];
		
	}
	foreach my $k(0..$#score)
	{
		$output3=$output3+$score[$k]*$wts[$#wts+2-($hnp+1)+$k];
		
	}
	$output1=(1/(1+exp(-$output1)));
	$output1=(int($output1*1000000))/1000000;
	$output2=(1/(1+exp(-$output2)));
	$output2=(int($output2*1000000))/1000000;
	$output3=(1/(1+exp(-$output3)));
	$output3=(int($output3*1000000))/1000000;

	$fin_pred=$temp[0]."\t".$output1."\t".$output2."\t".$output3."\n";
	$output1=0;
	$output2=0;
	$output3=0;

#close FINOUT;
}#end of if plant==1 block

if ($plant==0)
{
my @hu=();
my @posit=();
my @wts=();
	open (FILE,"$bindir/tables/nonplant_train.wts")||die "File chlor.wts does not exist";
	while (<FILE>)
	{
		if ($_=~/^\t(\d*)\t{2}(\d*)\t(-?\d*\.\d*e?-?\d*)\n/)
		{
			push @hu,$1;
			push @posit, $2;
			push @wts, $3;
			
		}
	}
	close FILE;
	
my @score=();
my $output1=0;
my $output2=0;
my $hnp=3;
my $inp=8;


#open (FINOUT,">$bindir/final_prediction")||die "Cannot open test file";


	my @temp =split (/\t/,$final);

	for (my $j=0 ; $j<$hnp ; $j++)
	{
		foreach my $k(0..$#temp-2)
		{
			if ($posit[$k+$j*$inp]==0)
			{
				$score[$j]=$wts[$k+$j*$inp];
			}
			else 
			{
				$score[$j]=$score[$j]+($temp[$k]*$wts[$k+$j*$inp]);
			}
		}
		
	}
	
	foreach my $k(0..$#score)
	{
		$score[$k]=((1/(1+exp(-$score[$k])))-(1/2));
		#print $_[0]=$score[$k],"\n";
	}
	
	
	
	$output1=$wts[$#wts+1-($hnp+1)*2];
	$output2=$wts[$#wts+1-($hnp+1)];
	foreach my $k(0..$#score)
	{
		$output1=$output1+$score[$k]*$wts[$#wts+2-($hnp+1)*2+$k];
		
	}
	foreach my $k(0..$#score)
	{
		$output2=$output2+$score[$k]*$wts[$#wts+2-($hnp+1)+$k];
		
	}
	
	$output1=(1/(1+exp(-$output1)));
	$output1=(int($output1*1000000))/1000000;
	$output2=(1/(1+exp(-$output2)));
	$output2=(int($output2*1000000))/1000000;
	
	$fin_pred=$temp[0]."\t".$output1."\t".$output2."\n";
	
	$output1=0;
	$output2=0;

#close FINOUT;
}#end of if plant==0
#print $fin_pred,"\n";

############################################################################################################################################################

############################################################################################################################################################
my $pred_t;
if ($plant==1)
{

my $chloroplast=1;
my $mitochondrion=1;
my $secreted=1;
my $cutoff=0.55;


#open (OUT,">$bindir/THATS_ALL_FOLKS!!")||die;

	$chloroplast=1;
	$mitochondrion=1;
	$secreted=1;

	if ($fin_pred=~/^(.*?)\t(\d*\.?\d*e?-?\d*)\t(\d\.?\d*e?-?\d*)\t(\d\.?\d*e?-?\d*)/ )
	{
	
		my $id=$1;
		my $chlsc=$2;
		
		my $mitsc=$3;
		
		my $spsc=$4;
		
		
		if ($chlsc=~/e/)
		{
			$chlsc=0;
		}
		if ($mitsc=~/e/)
		{
			$mitsc=0;
		}
		if ($spsc=~/e/)
		{
			$spsc=0;
		}
		if ($chlsc < $cutoff)
		{
			$chloroplast=0;
		}
		if ($mitsc < $cutoff)
		{
			$mitochondrion=0;
		}
		if ($spsc < $cutoff)
		{
			$secreted=0;
		}
		#print "$chloroplast\t$mitochondrion\t$secreted\n";
				if ($chloroplast==1 && $mitochondrion==0 && $secreted==0)
				{
					#print OUT "$id\t$chlsc\t$mitsc\t$spsc\tchloroplast\n";
					$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tchloroplast\n";
				}
				if ($chloroplast==0 && $mitochondrion==1 && $secreted==0)
				{
					#print OUT "$id\t$chlsc\t$mitsc\t$spsc\tmitochondrion\n";
					$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tmitochondrion\n";
				}
				
				if ($chloroplast==0 && $mitochondrion==0 && $secreted==1)
				{
					$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tsecreted\n";
					
					
				}
				if ($chloroplast==0 && $mitochondrion==1 && $secreted==1)
				{
					if ($mitsc > $spsc)
					{
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tmitochondrion\n";
						
					}
					else
					{	
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tsecreted\n";						
						
					}
				}
				if ($chloroplast==1 && $mitochondrion==0 && $secreted==1)
				{
					if ($chlsc > $spsc || $signal eq 'N')
					{
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tchloroplast\n";
						
					}
					else
					{	
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tsecreted\n";						
					}
				}
				
				if ($chloroplast==1 && $mitochondrion==1 && $secreted==0)
				{
					if ($chlsc > $mitsc)
					{
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tchloroplast\n";
						
					}
					else
					{
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tmitochondrion\n";
									
					}
				}
				if ($chloroplast==0 && $mitochondrion==0 && $secreted==0)
				{
					if ($signal eq 'Y')
					{
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tsecreted\n";
					}
					else
					{
						$pred_t="$id\t$chlsc\t$mitsc\t$spsc\tother\n";
					}

				}
				
			
		
	}

#
}#end of if $plant==1 block

if ($plant==0)
{

my $mitochondrion=1;
my $secreted=1;
my $cutoff=0.16;

#open (OUT,">$bindir/THATS_ALL_FOLKS!!")||die;



	$mitochondrion=1;
	$secreted=1;

	if ($fin_pred=~/^(.*?)\t(\d\.?\d*e?-?\d*)\t(\d\.?\d*e?-?\d*)\n/ )
	{
	
		my $id=$1;
		my $mitsc=$2;
		
		my $spsc=$3;
		
		
		if ($mitsc=~/e/)
		{
			$mitsc=0;
		}
		if ($spsc=~/e/)
		{
			$spsc=0;
		}
		if ($mitsc < $cutoff)
		{
			$mitochondrion=0;
		}
		if ($spsc < $cutoff)
		{
			$secreted=0;
		}
		
				
				if ($mitochondrion==1 && $secreted==0)
				{
					$pred_t="$id\t$mitsc\t$spsc\tmitochondrion\n";
					
				}
				
				if ($mitochondrion==0 && $secreted==1)
				{
					$pred_t="$id\t$mitsc\t$spsc\tsecreted\n";
					
				}
				if ($mitochondrion==1 && $secreted==1)
				{
					if ($mitsc > $spsc || $signal eq 'N')
					{
						$pred_t="$id\t$mitsc\t$spsc\tmitochondrion\n";
						
					}
					else
					{	
						$pred_t= "$id\t$mitsc\t$spsc\tsecreted\n";						
						

					}
				}
				
				if ($mitochondrion==0 && $secreted==0)
				{
					if ($signal eq 'Y')
					{	
						$pred_t= "$id\t$mitsc\t$spsc\tsecreted\n";						
						

					}
					else
					{
						$pred_t="$id\t$mitsc\t$spsc\tother\n";	
					}
				}
				
			
		
	}



}#end of if plant==0 block
#print $pred_t,"\n";
#########################################################################################################################################################

###############################################Get cleavage site prediction #############################################################################


my $stdev=5;

my $subloca;
my $prediction;


	if (($pred_t=~/^(.*?)\t.*\t.*\t.*\t(\w*)\n/)||($pred_t=~/^(.*?)\t.*\t.*\t(\w*)\n/))
	{
		$subloca=$2;
	}

	if ($subloca eq "chloroplast")
	{	
		$prediction=$chlor_prednp;
	}
	elsif ($subloca eq "mitochondrion")
	{
		$prediction=$mitoch_prednp;
	}
	elsif ($subloca eq "secreted")
	{
		$prediction=$sp_prednp;
	}
	elsif ($subloca eq "other")
	{
		$prediction="-";
	}
	




##########################################Determine the existence or not of an lTP#####################################################################

if ($plant)
{
	my $clst_all;
	my $sc_all;
	my $clst_c;
	my $sc_c;
	my $clst_d;
	my $sc_d;
	
open (IN,"$bindir/hmmres_lumen$pid")||die "hmmmitoch";
{
	while(<IN>)
	{
		if ($_=~/(.*?) \s*\d\/\d*\s*\d*\s*(\d*)\s[\[,\.][\.\]]\s*\d*\s*\d*\s[\[,\.][\.\]]\s*(-?\d*\.?\d?)\s.*/)
		{
			
			$clst_all=$2+26;
			$sc_all=$3;
		}
	}
}
close IN;


open (IN,"$bindir/hmmres_lumen_c$pid")||die "hmmmitoch";
{
	while(<IN>)
	{
		if ($_=~/(.*?) \s*\d\/\d*\s*\d*\s*(\d*)\s[\[,\.][\.\]]\s*\d*\s*\d*\s[\[,\.][\.\]]\s*(-?\d*\.?\d?)\s.*/)
		{
			$clst_c=$2+26;
			$sc_c=$3;
		}
	}
}
close IN;


open (IN,"$bindir/hmmres_lumen_d$pid")||die "hmmmitoch";
{
	while(<IN>)
	{
		if ($_=~/(.*?) \s*\d\/\d*\s*\d*\s*(\d*)\s[\[,\.][\.\]]\s*\d*\s*\d*\s[\[,\.][\.\]]\s*(-?\d*\.?\d?)\s.*/)
		{
			$clst_d=$2+26;
			$sc_d=$3;
		}
	}
}
close IN;
my $scq=0;
my $flag=0;
if ($subloca eq "chloroplast")
{
	
	if ($sc_all > 0)
	{
		$ltp=$clst_all;
		$scq=$sc_all;
		$flag=1;
		$thyl=1;
	}
	
	if ($sc_c > 0)
	{
		if ($sc_c > $scq)
		{
			$ltp=$clst_c;
			$scq=$sc_c;
		}
		$flag=1;
		$thyl=1;
	}
	if ($sc_d > 0)
	{
		if ($sc_d > $scq)
		{
			$ltp=$clst_d;
			$scq=$sc_d;
		}
		$flag=1;
		$thyl=1;
	}
	
		$scq=0;
		
}
}#end of if plant
#####################################################################################################################


#print "$id_a\t$subloca\t$prediction";
chomp $pred_t;
print $pred_t,"\t",$prediction,"\t";
if ($thyl)
{
		print "\tthylakoid\t".$ltp;
}
print "\n";

if ($graph)
{
	if ($subloca eq "chloroplast")
	{
		my @temp = split (/\t/, $pts_tst_chlor);
		if ($prediction-20 > 0)
		{
			foreach my $h ($prediction-20..$prediction+15)
			{
				my $star=int($temp[$h]*100)/3;
				print $h,"\t";
				print "*"x$star;
				print "*\n";
			}	
		}
		else
		{
			foreach my $h (1..$prediction+20)
			{
				my $star=int($temp[$h]*100)/3;
				print $h,"\t";
				print "*"x$star;
				print "*\n";
			}	
			last;
		}
	}
			
	if ($subloca eq "mitochondrion")
	{
		my @temp = split (/\t/,$pts_tst_mitoch);
		if ($prediction-20 > 0)
		{
			foreach my $h ($prediction-20..$prediction+15)
			{
				my $star=int($temp[$h]*100)/3;
				print $h,"\t";
				print "*"x$star;
				print "*\n";
			}
		}
		else
		{
			foreach my $h (1..$prediction+20)
			{
				my $star=int($temp[$h]*100)/3;
				print $h,"\t";
				print "*"x$star;
				print "*\n";
			}
		}
	}

	if ($subloca eq "secreted")
	{
		my $cu=0;
		my @temp = split (//,$sequence);
		if ($prediction-20 > 0)
		{
			foreach my $h ($prediction-20..$prediction+15)
			{
				foreach my $e ($h-9..$h+9)
				{
					$cu=$cu+$hydro{$temp[$e]};
				}
				print $h,"\t";
				$cu=$cu/2;
				print "*"x$cu;
				print "*\n";
			}
		}
		else
		{
			foreach my $h (1..$prediction+20)
			{
				foreach my $e ($h-9..$h+9)
				{
					$cu=$cu+$hydro{$temp[$e]};
					#print $cu,"\n";
				}
				print $h,"\t";
				$cu=$cu/2;
				print "*"x$cu;
				print "*\n";
			}
			last;
		}
	}
	print "\n\n";

}#endof if graph



#`rm $bindir/hmmres* $bindir/temp*-120 $bindir/jspp_*$bindir/predisi*`;
`rm $bindir/*$pid*`;
}#end of subpredsl
